% demo explaining the exact meaning of the parameters of Gabor functions
% stored in a .b files by the mp5 program for Matching Pursuit
% decomposition of signals. rkus@fuw.edu.pl and durka@fuw.edu.pl 2014

clear all;

% generate signal containing one Gabor

numberOfSamples   = 512;
samplingFrequency = 256.0;
amplitude         = 9.0;
width             = 128/samplingFrequency;  % width in seconds
frequency         = 30.0;                   % frequency in Hz
position          = 256/samplingFrequency;  % position in seconds
phase             = pi/3;                   % phase

t      = (0:1:numberOfSamples-1)/samplingFrequency;
signal = amplitude*exp(-pi.*((t-position)/width).^2).*cos(2*pi*frequency.*(t-position) + phase);

% save the signal as floats in a file demo.dat
% demo.cfg prepared for this file contains information about 
% numberOfSamples and samplingFrequency as above, plus parameters of the
% decomposition

file = fopen('demo.dat','wb');
fwrite(file, signal,'float32');
fclose(file);

fprintf('now run mp5 -f demo.set; this which will produce the file demo_smp.b')
% pause
% after that we read in the results of decomposition

[book header numberOfSamples] = readonebookv5('demo_smp.b',1,1);
numberOfAtoms = size(book.atoms,1);

% those are constans values determining the organization of the atoms
% parameters in *.b files 
TYPE      = 1;
MODULUS   = 2;
AMPLITUDE = 3;
POSITION  = 4;
SCALE     = 5;
FREQUENCY = 6;
PHASE     = 7;

reconstruction = zeros(1,numberOfSamples);
t              = (0:1:numberOfSamples-1)/header.samplingFrequency;

% reading results of decomposition snd plotting signal and reconstruction 
for atom=1:numberOfAtoms
    amplitudeReconstructed = book.atoms(atom,AMPLITUDE);
    positionReconstructed  = book.atoms(atom,POSITION)/header.samplingFrequency; 
    scaleReconstructed     = book.atoms(atom,SCALE)/header.samplingFrequency; 
    frequencyReconstructed = book.atoms(atom,FREQUENCY)*header.samplingFrequency/2.0; 
    % frequency is saved as a fraction of the Nyquist freq for given signal                                                                                  
    phaseReconstructed     = book.atoms(atom,PHASE);
    reconstruction = reconstruction + ...
        amplitudeReconstructed*exp(-pi.*((t-positionReconstructed)/scaleReconstructed).^2).*cos(2*pi*frequencyReconstructed.*(t-positionReconstructed) + phaseReconstructed);
end

% plotting
subplot(311)
plot(t,signal); 
title('simulated signal from demo.dat')
subplot(312)
plot(t,book.signal); 
title('signal stored in demo\_smp.b')
%this is signal read from the *.b file, should be the same as the original :-)
subplot(313)
plot(t,reconstruction,'r');
title(sprintf('signal reconstructed from %d atom(s) from MP decomposition', numberOfAtoms))

% energy check
Esignal=sum(book.signal.*book.signal);
Eatoms=sum(book.atoms(atom,MODULUS).^2);
fprintf('\nenergy of the signal: %.3f', Esignal);
fprintf('\n   energy of atom(s): %.3f\n', Eatoms);


